/// <reference types="lodash" />
import { ExprSource, LoadedPCData, NodeStyleInspection, VirtualFrame } from "paperclip-utils";
import { VirtualNode, EngineErrorEvent, EngineDelegateEvent, LoadedData } from "paperclip-utils";
import { Transform, Box, Point, Size } from "./geom";
export declare const DEFAULT_FRAME_BOX: {
    width: number;
    height: number;
    x: number;
    y: number;
};
export declare type Canvas = {
    showTools: boolean;
    panning: boolean;
    transform: Transform;
    size: Size;
    scrollPosition: Point;
    mousePosition?: Point;
};
export declare type BoxNodeInfo = {
    nodePath: string;
    box: Box;
};
export declare enum FSItemKind {
    FILE = "file",
    DIRECTORY = "directory"
}
export declare type File = {
    kind: FSItemKind.FILE;
    absolutePath: string;
    url: string;
    name: string;
};
export declare type Directory = {
    name: string;
    kind: FSItemKind.DIRECTORY;
    absolutePath: string;
    url: string;
    children: Array<FSItem>;
};
export declare type FSItem = File | Directory;
export declare type UIState = {
    pathname: string;
    query: Partial<{
        canvasFile: string;
        id: string;
        expanded: boolean;
        frame: number;
        embedded: boolean;
    }>;
};
export declare type Project = {
    name: string;
    documents: Record<string, string>;
};
export declare type SharedState = {
    documents: Record<string, string | Blob>;
};
export declare enum SyncLocationMode {
    None = 0,
    Query = 1,
    Location = 2
}
export declare type VirtualNodeSourceInfo = {
    virtualNodePath: number[];
    source: ExprSource;
};
export declare type DesignerState = {
    syncLocationMode?: number;
    canvasClickTimestamp?: number;
    ui: UIState;
    readonly: boolean;
    sharable: boolean;
    birdseyeFilter?: string;
    centeredInitial: boolean;
    toolsLayerEnabled: boolean;
    currentError?: EngineErrorEvent;
    showBirdseye?: boolean;
    loadedBirdseyeInitially?: boolean;
    loadingBirdseye?: boolean;
    resizerMoving?: boolean;
    mountedRendererIds: string[];
    currentEngineEvents: Record<string, EngineDelegateEvent[]>;
    allLoadedPCFileData: Record<string, LoadedData>;
    pcFileDataVersion: number;
    availableBrowsers: AvailableBrowser[];
    resourceHost: string;
    highlightNodePath: string;
    scopedElementPath?: string;
    selectedNodePaths: string[];
    expandedNodePaths: string[];
    selectedNodeSources: VirtualNodeSourceInfo[];
    selectedNodeStyleInspections: NodeStyleInspection[];
    projectDirectory?: Directory;
    metaKeyDown?: boolean;
    optionKeyDown?: boolean;
    canvas: Canvas;
    scrollSize?: Size;
    frameSize?: Size;
    boxes: Record<string, Box>;
    zoomLevel: number;
};
export declare type HistState = {
    shared: SharedState;
    history: {
        past: SharedState[];
        future: SharedState[];
    };
};
declare type ActionShape = {
    type: string;
    payload: any;
};
export declare type AppState = {
    actions: ActionShape[];
    shared: SharedState;
    designer: DesignerState;
    compact?: boolean;
} & HistState;
export declare enum EnvOptionKind {
    Public = "Public",
    Private = "Private",
    Browserstack = "Browserstack"
}
export declare type EnvOption = {
    kind: EnvOptionKind;
    launchOptions: any;
};
export declare type AvailableBrowser = {
    os: string;
    osVersion: string;
    browser: string;
    device: string;
    browserVersion: string;
};
export declare const INITIAL_STATE: AppState;
export declare const isExpanded: (designer: DesignerState) => boolean;
export declare const getActiveFrameIndex: (designer: DesignerState) => number;
export declare const IS_WINDOWS: boolean;
export declare const resetCanvas: (canvas: Canvas) => {
    scrollPosition: {
        x: number;
        y: number;
    };
    transform: {
        x: number;
        y: number;
        z: number;
    };
    showTools: boolean;
    panning: boolean;
    size: Size;
    mousePosition?: Point;
};
export declare const mergeBoxesFromClientRects: (boxes: Record<string, Box>, rects: Record<string, Box>) => {};
export declare const boxIntersectsPoint: (box: Box, point: Point) => boolean;
export declare const findBoxNodeInfo: (point: Point, boxes: Record<string, Box>) => BoxNodeInfo | null;
export declare const calcFrameBox: (rects: Record<string, Box>) => {
    x: number;
    y: number;
    width: number;
    height: number;
};
export declare const getFSItem: (absolutePath: string, current: FSItem) => any;
export declare const getSelectedFrames: (designer: DesignerState) => VirtualFrame[];
export declare const getFrameFromIndex: (frameIndex: number, designer: DesignerState) => VirtualFrame;
export declare const getNodeInfoAtPoint: (point: Point, transform: Transform, boxes: Record<string, Box>, expandedFrameIndex?: number) => BoxNodeInfo;
export declare const getScopedBoxes: (boxes: Record<string, Box>, scopedElementPath: string, root: VirtualNode) => Pick<Record<string, Box>, string>;
export declare const getFrameBoxes: (boxes: Record<string, Box>, frameIndex: number) => import("lodash").Dictionary<Box>;
export * from "./geom";
export declare const getPreviewChildren: (frame: VirtualNode) => VirtualNode[];
export declare const getCurrentPreview: (designer: DesignerState) => VirtualNode;
export declare const getCurrentPreviewFrameBoxes: (designer: DesignerState) => {
    width: number;
    height: number;
    x: number;
    y: number;
    title?: string;
    visible?: boolean;
}[];
/**
 * Called when file is data -- few conditions need to be met though:
 * canvas is loaded
 * file data is loaded
 */
export declare const maybeCenterCanvas: (designer: DesignerState, force?: boolean) => DesignerState;
export declare const updateShared: (state: AppState, shared: Partial<SharedState>) => {
    shared: {
        documents: Record<string, string | Blob>;
    };
    actions: ActionShape[];
    designer: DesignerState;
    compact?: boolean;
    history: {
        past: SharedState[];
        future: SharedState[];
    };
};
export declare const centerEditorCanvas: (designer: DesignerState, innerBounds?: Box, zoomOrZoomToFit?: boolean | number) => DesignerState;
export declare const getActivePCData: (designer: DesignerState) => LoadedPCData;
export declare const getAppActivePCData: (state: AppState) => LoadedPCData;
export declare const getSelectedNodePaths: (state: AppState) => string[];
export declare const getInspectionInfo: (state: AppState) => NodeStyleInspection[];
export declare const pruneDeletedNodes: (designer: DesignerState) => DesignerState;
